package de.szut.lf8_starter.project.action;

import de.szut.lf8_starter.project.ProjectEntity;
import de.szut.lf8_starter.project.ProjectMapper;
import de.szut.lf8_starter.project.ProjectService;
import de.szut.lf8_starter.project.dto.GetProjectDto;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.responses.ApiResponses;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

@RestController
@RequestMapping(value = "projects")
public class GetProjectAction {
    private final ProjectService projectService;
    private final ProjectMapper projectMapper;

    public GetProjectAction(ProjectService projectService, ProjectMapper projectMapper) {
        this.projectService = projectService;
        this.projectMapper = projectMapper;
    }

    @Operation(summary = "Find project by ID")
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "Project found", content = {
                    @Content(mediaType = "application/json", schema = @Schema(implementation = GetProjectDto.class))
            }),
            @ApiResponse(responseCode = "404", description = "Project not found", content = @Content)
    })
    @GetMapping("/{id}")
    public ResponseEntity<GetProjectDto> findArticleById(@PathVariable Long id) {
        ProjectEntity project = this.projectService.findById(id);

        return new ResponseEntity<>(this.projectMapper.mapToGetDto(project), HttpStatus.OK);
    }
}
